from bottle import Bottle, request
import waitress
from doreah import settings
from doreah.logging import log
from .database import db

PORT = settings.get_settings("PORT")
HOST = settings.get_settings("HOST")
THREADS = 12


server = Bottle()

@server.post("/<endpoint>")
def post_endpoint(endpoint):

	data = request.json
	log("Received request to endpoint " + endpoint)
	try:
		cls = db._getclassbyname(endpoint)
		log("Mapped to class " + str(cls))
		# instantiate a new object of this class
		cls(**data)
		db.save()
	except:
		log("Does not exist!")
		return 404



waitress.serve(server, host=HOST, port=PORT, threads=THREADS)



name = "myrcella"
version = 0,1
versionstr = ".".join(str(n) for n in version)
desc = "Simple API server to collect data according to dynamically loaded yaml specifications"
author = {
	"name": "Johannes Krattenmacher",
	"email": "python@krateng.dev",
	"github": "krateng"
}
requires = [
	"bottle>=0.12.16",
	"waitress>=1.3",
	"doreah>=1.3"
]
resources = [
	"data_files/*/*"
]







### DOREAH CONFIGURATION

from doreah import config
import pkg_resources

config(
	packageutils={
		"packagename": name,
		"populate": pkg_resources.resource_filename(__name__,"data_files")
	}
)
from doreah.packageutils import datafolder, init_datafolder
init_datafolder()
config(
	logging={
		"logfolder": datafolder("logs")
	},
	settings={
		"files":[
			datafolder("settings","default.ini"),
			datafolder("settings","settings.ini")
		]
	}
)



from doreah import database
from doreah.packageutils import datafolder
from doreah.regular import hourly
import yaml

import os


db = database.Database(file=datafolder("database.ddb"))

typenames = {
	"str":str,
	"int":int,
	"bool":bool,
	"float":float
}

#class Endpoint(db.DBObject):
#	__primary__ = "name",
#	name: str
#	data: dict = {}


@hourly
def load_endpoints():
	for fi in os.listdir(datafolder("endpoints")):
		try:
			with open(datafolder("endpoints",fi),"r") as f:
				info = yaml.safe_load(f)
				# create new endpoint or update existing
				#ep = Endpoint(name=data["name"],data=data["data"]) #data

				types = {}
				defaults = {}
				for var in info["data"]:

					val = info["data"][var]
					if type(val) in [dict,list]: continue
					if val in typenames:
						types[var] = typenames[val]
					else:
						types[var] = type(val)
						defaults[var] = val

				print(types)
				print(defaults)

				# this dynamically creates a subclass of DBObject with the default values and the types as annotations
				type(info["name"],(db.DBObject,),{**defaults,"__annotations__":types})

		except:
			raise
			print("Could not load endpoint information from",fi)

	db.save()
